;
; Hardware includes for Game Boy
;
; Copyright 2018 Damian Yerrick
; 
; This software is provided 'as-is', without any express or implied
; warranty.  In no event will the authors be held liable for any damages
; arising from the use of this software.
; 
; Permission is granted to anyone to use this software for any purpose,
; including commercial applications, and to alter it and redistribute it
; freely, subject to the following restrictions:
; 
; 1. The origin of this software must not be misrepresented; you must not
;    claim that you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation would be
;    appreciated but is not required.
; 2. Altered source versions must be plainly marked as such, and must not be
;    misrepresented as being the original software.
; 3. This notice may not be removed or altered from any source distribution.
;
  IF  !DEF(GB_INC)
GB_INC SET 1

; Now I have a dilemma: Do I want to replace this or not?
; https://github.com/tobiasvl/hardware.inc/blob/master/hardware.inc

; RAM areas ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; VRAM on the Game Boy consists of three 2K banks of CHR RAM and
; two 1K nametables.
CHRRAM0 EQU $8000  ; Sprite tiles 0-127; BG tiles 0-127 if BG_CHR01 on
CHRRAM1 EQU $8800  ; Sprite and BG tiles 128-255
CHRRAM2 EQU $9000  ; BG tiles 0-127 if BG_CHR01 off
NTRAM0  EQU $9800
NTRAM1  EQU $9C00
RAM     EQU $C000
RAM_END EQU $DFFF

OAM_    EQU $FE00
HRAM_   EQU $FF80  ; RIOT memory

; Controller ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

P1          EQU $FF00
P1_NONE     EQU $30
P1_BUTTONS  EQU $10
P1_DPAD     EQU $20
BKEY_A      EQU 0
KEY_A       EQU $01
BKEY_B      EQU 1
KEY_B       EQU $02
BKEY_SELECT EQU 2
KEY_SELECT  EQU $04
BKEY_START  EQU 3
KEY_START   EQU $08
BKEY_RIGHT  EQU 4
KEY_RIGHT   EQU $10
BKEY_LEFT   EQU 5
KEY_LEFT    EQU $20
BKEY_UP     EQU 6
KEY_UP      EQU $40
BKEY_DOWN   EQU 7
KEY_DOWN    EQU $80
; KEY_R    EQU $100
; KEY_L    EQU $200

; Serial port ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This will be filled in when someone solders me an NES controller
; to Game Link cable.

; Audio ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This will be filled in when I try porting Pently.

NR52    EQU $FF26
WAVERAM EQU $FF30

; LCD ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; 76543210
; |||||||+- Render BG
; ||||||+-- Render sprites
; |||||+--- 0: Sprites 8x8; 1: Sprites 8x16
; ||||+---- 0: Main BG uses BGMAP0; 1: Main BG uses BGMAP1
; |||+----- 0: BG uses CHRRAM2 and CHRRAM1; 1: BG uses CHRRAM0 and CHRRAM1
; ||+------ Enable window
; |+------- 0: Window uses BGMAP0; 1: Window uses BGMAP1
; +-------- LCD on
LCDC         EQU $FF40
OBJ_ON       EQU $02
OBJ_8X16     EQU $06
BG_NT0       EQU $01
BG_NT1       EQU $09
BG_CHR21     EQU $00
BG_CHR01     EQU $10
WINDOW_NT0   EQU $20
WINDOW_NT1   EQU $60
LCD_ON       EQU $80

; 76543210
;  |||||++- (RO) 0: hblank; 1: vblank; 3: OAM scan; 3: active
;  ||||+--- (RO) 1: LYC equals LY right now
;  |||+---- (RW) 1: assert STAT interrupt during hblank
;  ||+----- (RW) 1: assert STAT interrupt during vblank
;  |+------ (RW) 1: assert STAT interrupt during OAM scan
;  +------- (RW) 1: assert STAT interrupt during LYC match
STAT          EQU $FF41
STAT_MODE     EQU $03
STAT_HBLANK   EQU $00
STAT_VBLANK   EQU $01
STAT_OAM      EQU $02
STAT_ACTIVE   EQU $03
STAT_LYC      EQU $04
STAT_HIRQ     EQU $08
STAT_VIRQ     EQU $10
STAT_OAMIRQ   EQU $20
STAT_LYCIRQ   EQU $40

SCY   EQU $FF42     ; V scroll
SCX   EQU $FF43     ; H scroll
LY    EQU $FF44     ; V position of beam
LYC   EQU $FF45     ; when LY reaches this value, assert LYC in STAT

; Writing M triggers immediate 160 byte copy of a display list from
; M<<8 to OAM at $FE00.  CPU must be running in HRAM.
OAM_DMA   EQU $FF46  ; trigger immediate copy

BOAM_BGP1  EQU 4
BOAM_HFLIP EQU 5
BOAM_VFLIP EQU 6
BOAM_BACK  EQU 7

; 76543210
; ||||||++- backdrop (3: black, 2: dark gray, 1: light gray, 0: white)
; ||||++--- color 1
; ||++----- color 2
; ++------- color 3
BGP    EQU $FF47
OBP0   EQU $FF48
OBP1   EQU $FF49

; Window position (this is like sprites or SMS backgrounds in that
; increasing the value moves the image down and to the right,
; and top left is WX=7 WY=0)
WY     EQU $FF4A
WX     EQU $FF4B

; Interrupts ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; 76543210  Which events cause an interrupt
;    ||||+- 1: CALL $40 on vblank
;    |||+-- 1: CALL $48 on STAT
;    ||+--- 1: CALL $50 on timer
;    |+---- 1: CALL $58 on Game Link byte completion
;    +----- 1: CALL $60 on keypress
IRQE        EQU $FFFF  ; which interrupts cause RST $40-$60
IRQF        EQU $FF0F  ; which interrupts have occurred
IF_VBLANK   EQU $01
IF_STAT     EQU $02
IF_TIMER    EQU $04
IF_SERIAL   EQU $08
IF_KEY      EQU $10

; ROM header ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GBC_MONO EQU $00
GBC_DUAL EQU $80
GBC_ONLY EQU $C0
SGB_NONE EQU $00
SGB_SUPPORT EQU $03
CART_ROM_ONLY EQU $00
CART_MBC1 EQU $01
CART_MBC5 EQU $09
REGION_J EQU $00
REGION_UE EQU $01

; Mapper ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Lower nibble = $A enables RAM if present; others disable
MBC1_SRAM_EN  EQU $2000
MBC1_SRAM_OFF EQU $00
MBC1_SRAM_ON  EQU $0A

; Select 16 KiB ROM bank at $4000-$7FFF
; ($00 treated as $01)
MBC1_ROM_BANK EQU $2000

; Select 8 KiB ROM bank at $A000, or 512 KiB PRG ROM outer bank
MBC1_SRAM_BANK EQU $4000

; Sense of MBC1_SRAM_BANK
MBC1_SRAM_MODE EQU $6000
MBC1_BIG_ROM   EQU $00
MBC1_BIG_RAM   EQU $01

  ENDC