;
; Main program for 240p test suite
; Copyright 2018 Damian Yerrick
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License along
; with this program; if not, write to the Free Software Foundation, Inc.,
; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
;
include "src/gb.inc"
include "src/global.inc"

section "ram_main", WRAM0

section "rom_main", ROM0

; Double-colon these so that the header can see them
timer_handler::
serial_handler::
joypad_handler::
  reti

main::
  ; Avoid uninitialized read warnings for nmis and cur_keys
  xor a
  ld [nmis],a
  ; set all keys so that first read_pad returns none new
  dec a
  ld [cur_keys],a

  ; Set up vblank handler
  ld a,IF_VBLANK
  ldh [IRQE],a  ; enable IRQs
  xor a
  ldh [IRQF],a  ; Acknowledge any pending IRQs
  ei
  ld a,LCD_ON
  ld [stat_lcdc_value],a
  ld [vblank_lcdc_value],a
  call help_init

  if 1
    call activity_credits
  else
    call activity_stopwatch
    ld a,KEY_A|KEY_B|KEY_START|KEY_LEFT|KEY_RIGHT
    ld b,helpsect_to_do
    call helpscreen
  endc
.loop:
  ld a,KEY_A|KEY_START|KEY_DOWN|KEY_UP|KEY_LEFT|KEY_RIGHT
  ld b,helpsect_144p_test_suite_menu
  call helpscreen

  ; Get the list of handlers for this menu page
  ld de,page_one_handlers
  or a
  jr z,.not_page_two
    ld de,page_two_handlers
  .not_page_two:

  ; Save menu selection and get the index into the handler list
  ld a,[help_wanted_page]
  ld c,a
  ld a,[help_cursor_y]
  ld b,a
  push bc

  ; Start does About instead of what is selected
  ld hl,new_keys
  bit BKEY_START,[hl]
  jr z,.not_start
    call activity_about
    jr .skip_activity
  .not_start:

  call de_index_a
  call jp_hl
.skip_activity:

  ; Restore menu selection
  pop bc
  ld a,b
  ld [help_cursor_y],a
  ld a,c
  ld [help_wanted_page],a
  jr .loop
jp_hl:
  jp hl

page_one_handlers:
  dw activity_cps_grid
  dw activity_linearity
  dw activity_gray_ramp
  dw activity_solid_screen
  dw activity_sharpness
  dw activity_about
  dw activity_credits

page_two_handlers:
  dw lame_boy_demo  ; overscan
  dw activity_shadow_sprite
  dw activity_stopwatch
  dw lame_boy_demo  ; manual lag test
  dw activity_hillzone_scroll
  dw activity_kiki_scroll
  dw activity_grid_scroll
  dw activity_full_stripes
  dw lame_boy_demo  ; motion blur
  dw lame_boy_demo  ; stn smearing
  dw activity_backlight_zone
  dw lame_boy_demo  ; sound test
  dw lame_boy_demo  ; audio sync
